//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: AntForm.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;

namespace AntGP
{
	/// <summary>
	/// This is the main form for the ant simulator.
	/// </summary>
    [ComVisible(true)]
	public class AntForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TabControl AntTabControl;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>
        /// First tab in this application.
        /// This tab uses Genetic Programming to evolve new solutions to the food collection problem.
        /// </summary>
		AntEvolveProgramTab evolveTab = new AntEvolveProgramTab();

        /// <summary>
        /// Second tab in this application.
        /// This tab is used to examine any evolved solutions.
        /// The user can also load a canned solution or even write their own.
        /// </summary>
		AntProgramTab programTab = new AntProgramTab();

        /// <summary>
        /// Third tab in this application.
        /// In this tab, the user can see a ant solution program in action.
        /// The bulk of this tab is a graphical map where the ants will move around and collect food.
        /// </summary>
		AntSimulatorTab simulatorTab = new AntSimulatorTab();

		public AntForm()
		{
			InitializeComponent();

			TabPage tab;
			
			tab = new TabPage("Evolve your own Genetic Program");
			evolveTab.Location = new Point(0,0);
			tab.Size = evolveTab.Size;
            tab.BackColor = Color.Gainsboro;
			tab.Controls.Add(evolveTab);
			AntTabControl.TabPages.Add( tab );

			tab = new TabPage("View your evolved Program, a premade program, or make one by hand");
			programTab.Location = new Point(0,0);
			tab.Size = programTab.Size;
            tab.BackColor = Color.Gainsboro;
			tab.Controls.Add(programTab);
			AntTabControl.TabPages.Add( tab );

			tab = new TabPage("Test your program solution on a map");
			simulatorTab.Location = new Point(0,0);
			tab.Size = simulatorTab.Size;
            tab.BackColor = Color.Gainsboro;
			tab.Controls.Add(simulatorTab);
			AntTabControl.TabPages.Add( tab );

            evolveTab.AntForm = this;
            programTab.AntForm = this;
            simulatorTab.AntForm = this;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}        

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.AntTabControl = new System.Windows.Forms.TabControl();
            this.SuspendLayout();
            // 
            // AntTabControl
            // 
            this.AntTabControl.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.AntTabControl.Location = new System.Drawing.Point(0, 0);
            this.AntTabControl.Name = "AntTabControl";
            this.AntTabControl.SelectedIndex = 0;
            this.AntTabControl.Size = new System.Drawing.Size(808, 640);
            this.AntTabControl.TabIndex = 0;
            // 
            // AntForm
            // 
            //this.AutoScaleBaseSize = new System.Drawing.Size(6, 14);
            this.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(101)))), ((int)(((byte)(101)))), ((int)(((byte)(101)))));
            this.ClientSize = new System.Drawing.Size(808, 640);
            this.Controls.Add(this.AntTabControl);
            this.Font = new System.Drawing.Font("Arial", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.Name = "AntForm";
            this.Text = "Ant Behavior Genetic Program Simulation";
            this.ResumeLayout(false);

		}
		#endregion

        /// <summary>
        /// When running in IE, call this when the user leaves the page
        /// to make sure the background threads don't keep running.
        /// </summary>
        public void StopThreads()
        {
            if (evolveTab != null)
            {
                evolveTab.StopThreads();
            }
            if (simulatorTab != null)
            {
                simulatorTab.StopThreads();
            }
        }

        /// <summary>
        /// Retrieves the program displayed in the TreeView on the AntProgramTab object.
        /// </summary>
        /// <returns>The ant program displayed on the Program Tab.</returns>
        public Program GetProgramFromProgramTab()
        {
            return programTab.GetDisplayedProgram();
        }

        /// <summary>
        /// Retrieves the best evolved program from the AntEvolveProgramTab
        /// </summary>
        /// <returns>The evolved program from the Evolve Tab.</returns>
        public Program GetProgramFromEvolveTab()
        {
            return evolveTab.GetEvolvedProgram();
        }

        /// <summary>
        /// Retrieves the map displayed on the AntSimulatorTab
        /// </summary>
        /// <returns>The map displayed on the Simulator Tab.</returns>
        public Map GetMapFromSimulatorTab()
        {
            return simulatorTab.GetMap();
        }

        /// <summary>
        /// Retrieves the map used to by the Genetic Program to evolve a solution on the AntEvolveProgramTab.
        /// </summary>
        /// <returns>The map from the Evolve Tab.</returns>
        public Map GetMapFromEvolveTab()
        {
            return evolveTab.GetMap();
        }

        /// <summary>
        /// Retrieves the Genetic Program Options used by the Evolver on the AntEvolveProgramTab.
        /// </summary>
        /// <returns>The options from the Evolve Tab.</returns>
        public GeneticProgramOptions GetOptionsFromEvolveTab()
        {
            return evolveTab.GetOptions();
        }

        /// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new AntForm());
		}

	}
}
