//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: GeneticProgramOptions.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;

namespace AntGP
{
	/// <summary>
	/// This class stores all of the configurable options for the Genetic Program.
	/// </summary>
	public class GeneticProgramOptions
	{
        /// <summary>
        /// Default constructor.
        /// </summary>
		public GeneticProgramOptions()
		{
		}

        /// <summary>
        /// Private for the number of ant programs created by the evolver for the Genetic Program.
        /// </summary>
		private int populationSize;
        /// <summary>
        /// The number of ant programs created by the evolver for the Genetic Program.
        /// </summary>
		public int PopulationSize
		{
			set 
			{
				populationSize = value;
			}
			get 
			{
				return populationSize;
			}
		}

        /// <summary>
        /// Private for the number of programs in the tournament.
        /// </summary>
		private int tournamentSize;
        /// <summary>
        /// The number of programs in the tournament.
        /// Each turn, a given number of programs are chosen from the entire population.
        /// The best two programs from the tournament are used to make two new programs.
        /// Those two new programs replace the worst two programs from the tournament.
        /// </summary>
		public int TournamentSize
		{
			set 
			{
				tournamentSize = value;
			}
			get 
			{
				return tournamentSize;
			}
		}

        /// <summary>
        /// Private for the percent odds that any random node is a leaf node.
        /// </summary>
		private int percentLeafNode;
        /// <summary>
        /// While making the initial population, percent (0-100) that any one node in the program tree is a leaf node.
        /// A smaller number here will make a larger tree.
        /// Note: Any number under 50 can cause an infinite size program and a stack overflow, which is why there is also a maximum program size.
        /// A value of 100 will cause every program to have only three nodes.
        /// </summary>
		public int PercentLeafNode
		{
			set 
			{
				percentLeafNode = value;
			}
			get 
			{
				return percentLeafNode;
			}
		}

        /// <summary>
        /// Private for the approximate maximum program size when creating the initial population.
        /// </summary>
        private int maximumProgramSize;
        /// <summary>
        /// Approximate maximum program size when creating the initial population.
        /// Once this size is exceeded, all nodes created after that point will be leaf nodes.
        /// </summary>
        public int MaximumProgramSize
        {
            set
            {
                maximumProgramSize = value;
            }
            get
            {
                return maximumProgramSize;
            }
        }

        /// <summary>
        /// Private for the percent odds that a child program should be mutated after crossover.
        /// </summary>
		private int percentMutation;
        /// <summary>
        /// After the crossover operation, this is the percentage chance (0-100) that each child node will be mutated.
        /// </summary>
		public int PercentMutation
		{
			set 
			{
				percentMutation = value;
			}
			get 
			{
				return percentMutation;
			}
		}

        /// <summary>
        /// Private for the percent odds that mutation on a program should be greedy.
        /// </summary>
		private int percentGreedyMutation;
        /// <summary>
        /// When performing mutation, this is the percentage chance (0-100) that the mutation should be greedy.
        /// In greedy mutation, the mutation is rejected if it does not improve the fitness of the program.
        /// </summary>
		public int PercentGreedyMutation
		{
			set 
			{
				percentGreedyMutation = value;
			}
			get 
			{
				return percentGreedyMutation;
			}
		}

        /// <summary>
        /// Private for the value for each remaining piece of food when calculating fitness.
        /// </summary>
		private int fitnessRemainingFood;
        /// <summary>
        /// The value for each remaining piece of food (food not returned to the nest) when calculating fitness.
        /// When caclulating fitness, lower values are better.
        /// </summary>
		public int FitnessRemainingFood
		{
			set 
			{
				fitnessRemainingFood = value;
			}
			get 
			{
				return fitnessRemainingFood;
			}
		}

        /// <summary>
        /// Private for the value for each piece of food that was not discovered when calculating fitness.
        /// </summary>
		private int fitnessUndiscoveredFood;
        /// <summary>
        /// The value for each piece of food that was not discovered (never picked up) when calculating fitness.
        /// When caclulating fitness, lower values are better.
        /// </summary>
		public int FitnessUndiscoveredFood
		{
			set 
			{
				fitnessUndiscoveredFood = value;
			}
			get 
			{
				return fitnessUndiscoveredFood;
			}
		}

        /// <summary>
        /// Private for the value for each square of water that has not been turned into a dead ant or leaf on water.
        /// </summary>
        private int fitnessWaterRemaining;
        /// <summary>
        /// The value for each square of water that has not been turned into a dead ant or leaf on water.
        /// When caclulating fitness, lower values are better.
        /// </summary>
        public int FitnessWaterRemaining
        {
            set
            {
                fitnessWaterRemaining = value;
            }
            get
            {
                return fitnessWaterRemaining;
            }
        }

        /// <summary>
        /// Private for the value for the number of turns required to complete the program when calculating fitness.
        /// </summary>
        private int fitnessNumberOfTurns;
        /// <summary>
        /// Value for the number of turns required to complete the program when calculating fitness.
        /// If all of the food was not collected, then the number of turns will be the maximum number of turns configured.
        /// When caclulating fitness, lower values are better.
        /// </summary>
        public int FitnessNumberOfTurns
        {
            set
            {
                fitnessNumberOfTurns = value;
            }
            get
            {
                return fitnessNumberOfTurns;
            }
        }

        /// <summary>
        /// Private for the value for how the program size effects its fitness.
        /// </summary>
		private int fitnessProgramComplexity;
        /// <summary>
        /// Value for how program size is used when calculating fitness.
        /// When calculating fitness the following formula is used: FitnessProgramComplexity * (ProgramSize / FitnessProgramComplexityGrouping)
        /// </summary>
		public int FitnessProgramComplexity
		{
			set 
			{
				fitnessProgramComplexity = value;
			}
			get 
			{
				return fitnessProgramComplexity;
			}
		}

        /// <summary>
        /// Private for how large the program size needs to be to effect the fitness of the program.
        /// </summary>
		private int fitenssProgramComplexityGrouping;
        /// <summary>
        /// How large does the program need to be to effect its fitness.
        /// When calculating fitness the following formula is used: FitnessProgramComplexity * (ProgramSize / FitnessProgramComplexityGrouping)
        /// </summary>
		public int FitenssProgramComplexityGrouping
		{
			set 
			{
				fitenssProgramComplexityGrouping = value;
			}
			get 
			{
				return fitenssProgramComplexityGrouping;
			}
		}

        /// <summary>
        /// Private for the strength of the pheromones released by the ants.
        /// </summary>
		private int pheromoneStrength;
        /// <summary>
        /// Strength of the pheromones released by the ants.
        /// </summary>
		public int PheromoneStrength
		{
			set 
			{
				pheromoneStrength = value;
			}
			get 
			{
				return pheromoneStrength;
			}
		}

        /// <summary>
        /// Private for the speed that the pheromones spread.
        /// </summary>
		private int pheromoneDissipationSpeed;
        /// <summary>
        /// Speed (0-100) that the pheromones spread.
        /// Each turn, pheromones of strength PheromoneDissipationSpeed / 4 are spread from a given square on the map to the 4 adjacent squares.
        /// Pheromones eventually dissipate away due to round off error.
        /// </summary>
		public int PheromoneDissipationSpeed
		{
			set 
			{
				pheromoneDissipationSpeed = value;
			}
			get 
			{
				return pheromoneDissipationSpeed;
			}
		}

        /// <summary>
        /// Private for the number of ants created when the ant program starts to run.
        /// </summary>
		private int numberOfAnts;
        /// <summary>
        /// The number of ants created when the ant program starts to run.
        /// </summary>
		public int NumberOfAnts
		{
			set 
			{
				numberOfAnts = value;
			}
			get 
			{
				return numberOfAnts;
			}
		}

        /// <summary>
        /// Private for the number of times each ant will run the ant program.
        /// </summary>
		private int numberOfTurnsPerAnt;
        /// <summary>
        /// The number of times each ant will run the ant program.
        /// </summary>
		public int NumberOfTurnsPerAnt
		{
			set 
			{
				numberOfTurnsPerAnt = value;
			}
			get 
			{
				return numberOfTurnsPerAnt;
			}
		}

        /// <summary>
        /// Limit for the number of times an ant can move.
        /// This calculation is done to prevent programs that have lots of move statements from appearing to perform because they
        /// seem to find the food faster.
        /// </summary>
		public long MaximumMovementForAllAntsPerTest
		{
			get
			{
                return (long)NumberOfTurnsPerAnt * (long)NumberOfAnts * 2L;
			}
		}

        /// <summary>
        /// Private for the percent odds that an ant will give up waiting for other ants to help lift food.
        /// </summary>
		private int waitForHelpThisTurnPercent;
        /// <summary>
        /// The percent odds (0-100) that an ant will give up waiting for other ants to help lift food.
        /// </summary>
		public int WaitForHelpThisTurnPercent
		{
			set 
			{
				waitForHelpThisTurnPercent = value;
			}
			get 
			{
				return waitForHelpThisTurnPercent;
			}
		}

        /// <summary>
        /// Private should the ants automatically drop the food when they are over the nest.
        /// </summary>
        private bool automaticallyDropFood;
        /// <summary>
        /// Should the ants automatically drop the food when they are over the nest.
        /// </summary>
        public bool AutomaticallyDropFood
        {
            set
            {
                automaticallyDropFood = value;
            }
            get
            {
                return automaticallyDropFood;
            }
        }

        /// <summary>
        /// Private for the list of non-leaf statements that can be included in the Genetic Programs population of ant programs.
        /// </summary>
        private List<Tokens.Instruction> nonLeafInstructionSet = new List<Tokens.Instruction>();
        /// <summary>
        /// The list of non-leaf statements that can be included in the Genetic Programs population of ant programs.
        /// </summary>
        public List<Tokens.Instruction> NonLeafInstructionSet
        {
            set
            {
                nonLeafInstructionSet = value;
            }
            get
            {
                return nonLeafInstructionSet;
            }
        }

        /// <summary>
        /// Private for the list of leaf statements that can be included in the Genetic Programs population of ant programs.
        /// </summary>
        private List<Tokens.Instruction> leafInstructionSet = new List<Tokens.Instruction>();
        /// <summary>
        /// The list of leaf statements that can be included in the Genetic Programs population of ant programs.
        /// </summary>
        public List<Tokens.Instruction> LeafInstructionSet
		{
			set 
			{
                leafInstructionSet = value;
			}
			get 
			{
                return leafInstructionSet;
			}
		}

        /// <summary>
        /// Private copy of the map that is used in the Genetic Program.
        /// </summary>
        private Map map;
        /// <summary>
        /// The map used in the Genetic Program.
        /// </summary>
        public Map Map
        {
            set
            {
                map = value;
            }
            get
            {
                return map;
            }
        }

        /// <summary>
        /// Random number generator to use when creating the initial population of ant programs.
        /// </summary>
        private Random initialPopulationRandomGenerator = new Random();

        /// <summary>
        /// Seed value for the initial population random number generator.
        /// </summary>
        public int InitialPopulationSeed
        {
            set
            {
                initialPopulationRandomGenerator = new Random(value);
            }
        }

        /// <summary>
        /// Returns a random number for use when creating the initial population.
        /// </summary>
        /// <param name="n">The maximum value of the random number.</param>
        /// <returns>The random number.</returns>
        public int InitialPopulationRandom(int n)
        {
            return initialPopulationRandomGenerator.Next(n);
        }

        /// <summary>
        /// Random number generator to use when interpreting the ant programs.
        /// </summary>
        private Random interpreterRandomGenerator = new Random();

        /// <summary>
        /// Seed value for the interpreter random number generator.
        /// </summary>
        public int InterpreterSeed
        {
            set
            {
                interpreterRandomGenerator = new Random(value);
            }
        }

        /// <summary>
        /// Returns a random number for use when creating interpreting the ant program..
        /// </summary>
        /// <param name="n">The maximum value of the random number.</param>
        /// <returns>The random number.</returns>
        public int InterpreterRandom(int n)
        {
            return interpreterRandomGenerator.Next(n);
        }
	}
}
