//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: Map.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Drawing;
using System.IO;
using System.Collections;

namespace AntGP
{
	/// <summary>
	/// The map contains the location of all the map items and the pheromones.  `
	/// </summary>
	public class Map
	{
        /// <summary>
        /// MapItem array that is the size of the map.
        /// </summary>
		private MapItem.Item[,] mapItems;

        /// <summary>
        /// The size of the map.
        /// </summary>
		private Size mapSize;

        /// <summary>
        /// Pheromone array the size of the map.
        /// </summary>
		private int[,] pheromones;	// the rounding error on the integer is what causes pheromone dissipation

        /// <summary>
        /// Location of the nest on the map.
        /// </summary>
        private Point nestLocation = NullPoint;

        /// <summary>
        /// List of all the food objects on the map. Note, this food is also in the mapItems array, but it must also be stored
        /// here so that we can store additional information about each piece of food, and so we can keep track of it while an
        /// ant is carrying it.
        /// </summary>
		private AllFood food = new AllFood();

        /// <summary>
        /// Cache of all the map item images that are drawn on the map.
        /// </summary>
		private Icons icons = new Icons();

        /// <summary>
        /// Some premade maps that I already found solutions for.
        /// </summary>
		public enum CannedMaps
		{
            /// <summary>Three piles of food, each piece of food has a weight of 1.</summary>
			ThreeFood1Piles,
            /// <summary>Six piles of food, each pile has a different weight 1-6.</summary>
			SixDifferentWeightFoodPiles,
            /// <summary>Seven pieces of food of weight 6..</summary>
			SevenFood6Pieces,
            /// <summary>Ten pieces of food, each piece of food has a weight of 9.</summary>
			TenFood9Pieces,
            /// <summary>One pile of food, each piece of food has a weight of 1, and a river between the nest and the food.</summary>
            OneFood1PileWithRiver,
            /// <summary>Three piles of food, each piece of food has a weight of 1, a river is between the nest and the food, and there are trees on both sides of the river..</summary>
            ThreeFood1PileWithRiverAndTree,
            /// <summary>Six pieces of food all weight 5, some food in the open, some partially blocked by a river, and some completely blocked by a river.</summary>
			SixFood5WithWallAndRiver
		}

        /// <summary>
        /// Default constructor makes a empty 40x40 map.
        /// </summary>
		public Map()
		{
			mapSize = new Size( 40, 40 );
			mapItems = new MapItem.Item[ mapSize.Width, mapSize.Height ];
			pheromones = new int[ mapSize.Width, mapSize.Height ];

			ClearMap();
		}

        /// <summary>
        /// Constructor that makes a map of a given size.
        /// </summary>
        /// <param name="size"></param>
		public Map(Size size)
		{
			mapSize = size;
            mapItems = new MapItem.Item[mapSize.Width, mapSize.Height];
			pheromones = new int[ mapSize.Width, mapSize.Height ];
			ClearMap();
		}

        /// <summary>
        /// Constructor that makes a new map based on an existing map.
        /// This is useful when running the Genetic Program, because running the interpreter on the map will alter it by moving the food.
        /// Should make a copy of the map before using the interpreter on it.
        /// </summary>
        /// <param name="map">The map to copy.</param>
		public Map(Map map)
		{
			mapSize = map.mapSize;
            mapItems = new MapItem.Item[mapSize.Width, mapSize.Height];
			pheromones = new int[ mapSize.Width, mapSize.Height ];

			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					AddItem( col, row, map.mapItems[col, row] );
				}
			}
			
			ClearPheromones();
		}

        /// <summary>
        /// Constructor that loads a map from a file.
        /// </summary>
        /// <param name="fileName">Name of the file to load the map from.</param>
		public Map(string fileName)
		{
			OpenMapFile(fileName);
			ClearPheromones();
		}

        /// <summary>
        /// Constructor that creates a map based on a hardcoded canned map.
        /// </summary>
        /// <param name="cannedMap">The canned map to create.</param>
		public Map(CannedMaps cannedMap)
		{
			LoadCannedMap( cannedMap );
		}

        /// <summary>
        /// Generic point that is off the map.
        /// </summary>
        public static Point NullPoint
        {
            get
            {
                return new Point(-1, -1);
            }
        }

        /// <summary>
        /// The size of the map in squares.
        /// </summary>
        public Size MapSize
        {
            get
            {
                return mapSize;
            }
        }

        /// <summary>
        /// The location of the nest within the map.
        /// </summary>
        public Point NestLocation
        {
            get
            {
                return nestLocation;
            }
        }

        /// <summary>
        /// Has all the food on the map been collected. If so, we can end the interpreter.
        /// </summary>
        /// <returns>True if all food has been collected.</returns>
        public bool IsAllFoodCollected()
        {
            return food.IsAllFoodCollected();
        }

        /// <summary>
        /// How much food has not been collected (returned to the nest).
        /// </summary>
        /// <returns>The amount of food that has not been returned to the nest.</returns>
        public int FoodNotCollected()
        {
            return food.FoodNotCollected();
        }
        
        /// <summary>
        /// How much food has not been located (never picked up)
        /// </summary>
        /// <returns>The amount of food that has not been picked up.</returns>
        public int FoodNotLocated()
        {
            return food.FoodNotLocated();
        }

        /// <summary>
        /// The amount of water squares left on the map that have not been chagned into dead ants or a leaf on water.
        /// </summary>
        /// <returns>The amount of water remaining.</returns>
        public int WaterRemaining()
        {
            int waterCount = 0;

            for (int col = 0; col < mapSize.Width; col++)
            {
                for (int row = 0; row < mapSize.Height; row++)
                {
                    if (mapItems[col, row] == MapItem.Item.Water)
                    {
                        waterCount++;
                    }
                }
            }

            return waterCount;
        }

        /// <summary>
        /// Reset certain variables on the food that are used to coordinate multiple ants carrying the same food.
        /// </summary>
        public void ResetFoodForNewTurn()
        {
            food.ResetFoodForNewTurn();
        }

        /// <summary>
        /// What items is at a given location on the map.
        /// </summary>
        /// <param name="location">The location to look at.</param>
        /// <returns>What is on the map at the given location.</returns>
        public MapItem.Item ItemAtLocation(Point location)
        {
            return mapItems[location.X, location.Y];
        }

        /// <summary>
        /// Is there food (of any weight) at the given map location.
        /// </summary>
        /// <param name="location">The location to look for food at.</param>
        /// <returns>The food that is at the map location, or null if there is no food there.</returns>
        public Food FoodAtLocation(Point location)
        {
            foreach (Food foodPiece in food)
            {
                if ((foodPiece.Location == location) && (!foodPiece.HasBeenCollected) && (foodPiece.AntsLifting.Count < foodPiece.Weight))
                {
                    return foodPiece;
                }
            }
            return null;
        }

        /// <summary>
        /// When reading map files, converts a character to a Map Item.
        /// </summary>
        /// <param name="c">The character from the map file to convert.</param>
        /// <returns>The Map Item that corresponds to that character, or nothing if it does not match any map item.</returns>
		private static MapItem.Item GetMapItem(char c)
		{
			switch( c )
			{
				case ' ': return MapItem.Item.Nothing;
                case 'O': return MapItem.Item.Obstacle;
                case 'W': return MapItem.Item.Water;
                case 'N': return MapItem.Item.Nest;
                case 'T': return MapItem.Item.Tree;
                case '1': return MapItem.Item.Food1;
                case '2': return MapItem.Item.Food2;
                case '3': return MapItem.Item.Food3;
                case '4': return MapItem.Item.Food4;
                case '5': return MapItem.Item.Food5;
                case '6': return MapItem.Item.Food6;
                case '7': return MapItem.Item.Food7;
                case '8': return MapItem.Item.Food8;
                case '9': return MapItem.Item.Food9;
			}

			return MapItem.Item.Nothing;
		}

        /// <summary>
        /// Opens a map file and reads each characacter from the file and converts it into a map item.
        /// </summary>
        /// <param name="fileName"></param>
		public void OpenMapFile(string fileName)
		{
			StreamReader reader = new StreamReader( fileName );
			string data = reader.ReadToEnd();
			reader.Close();

			string[] lines = data.Split('\n');

            // Just in case the lines in the file are a different length (easily possible because nothing is represented by a space)
            // Then find the length of the longest line and use that as the width of the map.
			int rows = lines.Length;
			int cols = 0;
			foreach( string line in lines )
			{
				if (line.Length > cols)
				{
					cols = line.Length;
				}
			}

			mapSize = new Size( cols, rows );
			mapItems = new MapItem.Item[rows,cols];
			pheromones = new int[ mapSize.Width, mapSize.Height ];

            nestLocation = NullPoint;

            // for each character on each line, find the map item that corresponds to that character.
			int row = 0;
			foreach( string line in lines )
			{
				int col = 0;
				foreach( char c in line )
				{
					AddItem( col, row, Map.GetMapItem(c));

					mapItems[col,row] = Map.GetMapItem(c);

					col++;
				}
				row++;
			}
		}

        /// <summary>
        /// Adds an item to the map. This also maintains the food collection and the nest location.
        /// Will handle removing food from the collection if it is replaced and making sure there is only one nest.
        /// Note: This function should be used to create the map, and should not be used to alter the map when running the interpreter.
        /// </summary>
        /// <param name="col">X location in the map.</param>
        /// <param name="row">Y location in the map.</param>
        /// <param name="item">The item to place on the map.</param>
		public void AddItem( int col, int row, MapItem.Item item )
		{
			if (( col < 0 ) || ( row < 0 ) || ( col >= mapSize.Width ) || ( row >= mapSize.Height ))
			{
				return;
			}

			if ( MapItem.IsFood( mapItems[col, row] ))
			{
				food.Remove( new Point( col, row ));	// already was food at this position. remove it.
			}
			if ( mapItems[col, row] == MapItem.Item.Nest )
			{
                nestLocation = NullPoint;				// Replacing the nest with something else
			}

			mapItems[col, row] = item;

            if (MapItem.IsFood(item))
			{
                food.Add(new Food(new Point(col, row), item));
			}
			if ( item == MapItem.Item.Nest )
			{
                if (this.nestLocation != NullPoint)
				{
					mapItems[nestLocation.X,nestLocation.Y] = MapItem.Item.Nothing;	// only one nest per map
				}
				this.nestLocation = new Point(col,row);
			}
		}

        /// <summary>
        /// Adds an item to the map. This also maintains the food collection and the nest location.
        /// Will handle removing food from the collection if it is replaced and making sure there is only one nest.
        /// Note: This function should be used to create the map, and should not be used to alter the map when running the interpreter.
        /// </summary>
        /// <param name="point">The location on the map to place the item.</param>
        /// <param name="item">The item to place on the map.</param>
        public void AddItem(Point point, MapItem.Item item)
        {
            this.AddItem(point.X, point.Y, item);
        }

        /// <summary>
        /// Changes a water square on the map to a dead ant.
        /// </summary>
        /// <param name="point">The location of the dead ant.</param>
        public void ReplaceWaterWithDeadAnt(Point point)
        {
            if (ItemAtLocation(point) == MapItem.Item.Water)
            {
                mapItems[point.X, point.Y] = MapItem.Item.DeadAnt;
            }
        }

        /// <summary>
        /// Changes a water square on the map with a leaf.
        /// </summary>
        /// <param name="point">The location of the leaf on water.</param>
        public void ReplaceWaterWithLeaf(Point point)
        {
            if (ItemAtLocation(point) == MapItem.Item.Water)
            {
                mapItems[point.X, point.Y] = MapItem.Item.LeafOnWater;
            }
        }

        /// <summary>
        /// Removes the food from the map, and replaces it with what was under the food.
        /// Ex: Ant moves into water and dies dropping the carried food. Under the food is a dead ant.
        /// </summary>
        /// <param name="point">The location of the food.</param>
        /// <param name="food">The food to pick up.</param>
        public void PickUpFood(Point point, Food food)
        {
            mapItems[point.X, point.Y] = food.ItemUnderFood;
            food.Location = NullPoint;
        }

        /// <summary>
        /// Drop food on the map.
        /// </summary>
        /// <param name="point">The location of the food.</param>
        /// <param name="food">The food to drop.</param>
        public void DropFood(Point point, Food food)
        {
            MapItem.Item item = ItemAtLocation(point);

            if (food != null)
            {
                if (item == MapItem.Item.Nest)
                {
                    food.HasBeenCollected = true;
                }

                if ((item == MapItem.Item.Nothing) || (item == MapItem.Item.DeadAnt))
                {
                    food.ItemUnderFood = item;
                    food.Location = point;
                    mapItems[point.X, point.Y] = food.FoodMapItem;
                }
            }
        }

        /// <summary>
        /// Pick up a leaf off of the map.
        /// </summary>
        /// <param name="point">The location of the leaf.</param>
        public void PickUpLeaf(Point point)
        {
            if (ItemAtLocation(point) == MapItem.Item.Leaf)
            {
                mapItems[point.X, point.Y] = MapItem.Item.Nothing;
            }
        }

        /// <summary>
        /// Drop a leaf on the map.
        /// </summary>
        /// <param name="point">The location of the leaf</param>
        public void DropLeaf(Point point)
        {
            MapItem.Item item = ItemAtLocation(point);

            if (item == MapItem.Item.Nothing)
            {
                mapItems[point.X, point.Y] = MapItem.Item.Leaf;
            }
            else if (item == MapItem.Item.Water)
            {
                mapItems[point.X, point.Y] = MapItem.Item.LeafOnWater;
            }
        }
		
        /// <summary>
        /// Clears everything from the map, including pheromones.
        /// </summary>
		public void ClearMap()
		{
			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					mapItems[col, row] = MapItem.Item.Nothing;
					pheromones[col, row] = 0;
				}
			}

            nestLocation = NullPoint;
			food.Clear();
		}

        /// <summary>
        /// Removes just the pheromones from the map.
        /// </summary>
		public void ClearPheromones()
		{
			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					pheromones[col, row] = 0;
				}
			}
		}

        /// <summary>
        /// Spreads the pheromones around the map based on the given percentage.
        /// For each square on the map, (pheromoneSpreadPercent / 4) % of that squares pheromones is transferred to the 4 adjacent squares.
        /// The pheromones will eventuall completely dissipate due to rounding off error.
        /// </summary>
        /// <param name="pheromoneSpreadPercent"></param>
		public void SpreadPheromones( int pheromoneSpreadPercent )
		{
            // Need a temporary copy of the pheromones so that transferring from square A to B doesn't effect the transfer of pheromones from B to A.
			int[,] tempPheromone = new int[mapSize.Width, mapSize.Height];
			int pheromone;

			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					tempPheromone[col, row] = 0;
				}
			}

			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					pheromone = pheromones[col,row];
					if ( pheromone > 0 )
					{	// Divide by 100 to make a percent. Divide by 400 becuase spread is across all 4 cells
						tempPheromone[ col, row ] += pheromone * ( 100 - pheromoneSpreadPercent ) / 100 ;
						if ( row > 0 )
						{
							tempPheromone[ col, row-1 ] += pheromone * pheromoneSpreadPercent / 400 ;
						}
						if ( row < mapSize.Height-1 )
						{
							tempPheromone[ col, row+1 ] += pheromone * pheromoneSpreadPercent / 400 ;
						}
						if ( col > 0 )
						{
							tempPheromone[ col-1, row ] += pheromone * pheromoneSpreadPercent / 400 ;
						}
						if ( col < mapSize.Width-1 )
						{
							tempPheromone[ col+1, row ] += pheromone * pheromoneSpreadPercent / 400 ;
						}
					}
				}
			}

            // Copy the temporary pheromone map into the main one.
			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
					pheromones[col, row] = tempPheromone[col, row];
				}
			}
		}

        /// <summary>
        /// Release pheromones on a given square on the map.
        /// </summary>
        /// <param name="location">The location to release the pheromones.</param>
        /// <param name="pheromoneStrength">The strength of the pheromones.</param>
        public void AddPheromone(Point location, int pheromoneStrength)
        {
            pheromones[location.X, location.Y] += pheromoneStrength;
        }

        /// <summary>
        /// Gets the strength of the pheromones at a given locaiton on the map.
        /// </summary>
        /// <param name="location">The location of the pheromones.</param>
        /// <returns>The strength of the pheromones at the given location.</returns>
        public int PheromoneStrength(Point location)
        {
            return pheromones[location.X, location.Y];
        }

        /// <summary>
        /// Gets the pheromones strength for a square that is one square adjacent to a given square.
        /// </summary>
        /// <param name="location">The location of the first square on the map.</param>
        /// <param name="direction">The direction of the adjacent square to get the phoremone strength of.</param>
        /// <returns>The strength of the pheromones.</returns>
        public int PheromoneStrength(Point location, Compass.CompassDirection direction)
        {
            Point newLocation = Compass.AdjacentLocation(location, mapSize, direction);
            if (newLocation == NullPoint)
            {
                return 0;
            }
            else {
                return PheromoneStrength(newLocation);
            }
        }

        /// <summary>
        /// Add a pile of food, water... to the map. Used by the canned map generator.
        /// </summary>
        /// <param name="startCol">X coordinate of the top left corner of the pile.</param>
        /// <param name="startRow">Y coordinate of the top left corner of the pile.</param>
        /// <param name="endCol">X coordinate of the bottom right corner of the pile.</param>
        /// <param name="endRow">Y coordinate of the bottom right corner of the pile.</param>
        /// <param name="item">The item to place on the map.</param>
		private void AddPile( int startCol, int startRow, int endCol, int endRow, MapItem.Item item )
		{
			if (( startCol < 0 ) || ( startRow < 0 ) || ( startCol >= mapSize.Width ) || ( startRow >= mapSize.Height ))
			{
				throw new ArgumentException("Start Location is not within map");
			}
			if (( endCol < 0 ) || ( endRow < 0 ) || ( endCol >= mapSize.Width ) || ( endRow >= mapSize.Height ))
			{
				throw new ArgumentException("End Location is not within map");
			}
			if (( startCol > endCol ) || ( startRow > endRow ))
			{
				throw new ArgumentException("Start and End Locations are reversed");
			}

			for ( int x = startCol; x <= endCol; x++ )
			{
				for ( int y = startRow; y <= endRow; y++ )
				{
					AddItem( x, y, item );
				}
			}
		}

        /// <summary>
        /// Add a line of food, water... to the map. Used by the canned map generator.
        /// </summary>
        /// <param name="startCol">X coordinate of the top left corner of the pile.</param>
        /// <param name="startRow">Y coordinate of the top left corner of the pile.</param>
        /// <param name="endCol">X coordinate of the bottom right corner of the pile.</param>
        /// <param name="endRow">Y coordinate of the bottom right corner of the pile.</param>
        /// <param name="item">The item to place on the map.</param>
        /// <param name="lineWidth">The width of the line of items to add</param>
		private void AddLine( int startCol, int startRow, int endCol, int endRow, MapItem.Item item, int lineWidth )
		{
			if (( startCol < 0 ) || ( startRow < 0 ) || ( startCol >= mapSize.Width ) || ( startRow >= mapSize.Height ))
			{
				throw new ArgumentException("Start Location is not within map");
			}
			if (( endCol < 0 ) || ( endRow < 0 ) || ( endCol >= mapSize.Width ) || ( endRow >= mapSize.Height ))
			{
				throw new ArgumentException("End Location is not within map");
			}

			int x,y;
			int rowDiff = endRow - startRow;
			int totalRows = Math.Abs( rowDiff ) + 1;
			int rowDirection = Math.Sign( rowDiff );
			int colDiff = endCol - startCol;
			int totalCols = Math.Abs( colDiff ) + 1;
			int colDirection = Math.Sign( colDiff );
			double rowsPerCol = (double) totalRows / (double) totalCols;

			for ( int i = 0; i < lineWidth; i++ )
			{
				for ( int colNum = 0; colNum < totalCols; colNum++ )
				{
					for ( int rowNum = 0; rowNum < rowsPerCol; rowNum++ )
					{
						x = startCol + colNum * colDirection;
						y = startRow + (int) Math.Floor( colNum * rowsPerCol + rowNum ) * rowDirection;

						if (( x >= 0 ) && ( y >= 0 ) && ( x < mapSize.Width ) && ( y < mapSize.Height ))
						{
							AddItem( x, y, item );
						}
					}
				}
				if ( Math.Abs( startCol - endCol ) < Math.Abs( startRow - endRow ))
				{
					startCol++;
					endCol++;
				}
				else
				{
					startRow++;
					endRow++;
				}
			}
		}

        /// <summary>
        /// Load a premade map into the system.
        /// </summary>
        /// <param name="mapType">The type of canned map.</param>
		public void LoadCannedMap( CannedMaps mapType )
		{
			mapSize = new Size(40, 40);
			mapItems = new MapItem.Item[ mapSize.Width, mapSize.Height ];
			pheromones = new int[ mapSize.Width, mapSize.Height ];
			ClearMap();

			switch ( mapType )
			{
				case CannedMaps.ThreeFood1Piles:
					AddItem( 5, 5, MapItem.Item.Nest );
                    AddPile(24, 10, 26, 11, MapItem.Item.Food1);
                    AddPile(25, 19, 28, 22, MapItem.Item.Food1);
                    AddPile(7, 30, 9, 31, MapItem.Item.Food1);
					break;
				case CannedMaps.SixDifferentWeightFoodPiles:
                    AddItem(5, 5, MapItem.Item.Nest);
                    AddPile(20, 2, 21, 2, MapItem.Item.Food3);
                    AddPile(27, 13, 28, 13, MapItem.Item.Food1);
                    AddItem(27, 23, MapItem.Item.Food6);
                    AddItem(10, 32, MapItem.Item.Food3);
					break;
				case CannedMaps.SevenFood6Pieces:
                    AddItem(10, 15, MapItem.Item.Nest);
                    AddItem(2, 3, MapItem.Item.Food6);
                    AddItem(21, 3, MapItem.Item.Food6);
                    AddItem(25, 13, MapItem.Item.Food6);
                    AddItem(2, 24, MapItem.Item.Food6);
                    AddItem(25, 25, MapItem.Item.Food6);
                    AddItem(10, 30, MapItem.Item.Food6);
                    AddItem(24, 34, MapItem.Item.Food6);
					break;
				case CannedMaps.TenFood9Pieces:
                    AddItem(20, 20, MapItem.Item.Nest);
                    AddItem(28, 5, MapItem.Item.Food9);
                    AddItem(3, 8, MapItem.Item.Food9);
                    AddItem(15, 8, MapItem.Item.Food9);
                    AddItem(7, 12, MapItem.Item.Food9);
					AddItem( 25, 12, MapItem.Item.Food9 );
                    AddItem(3, 25, MapItem.Item.Food9);
                    AddItem(24, 26, MapItem.Item.Food9);
                    AddItem(31, 27, MapItem.Item.Food9);
                    AddItem(8, 28, MapItem.Item.Food9);
                    AddItem(23, 37, MapItem.Item.Food9);
					break;
                case CannedMaps.OneFood1PileWithRiver:
                    AddItem(5, 5, MapItem.Item.Nest);
                    AddLine(0, 39, 39, 0, MapItem.Item.Water, 2);
                    AddPile(27, 24, 30, 27, MapItem.Item.Food1);
                    break;
                case CannedMaps.ThreeFood1PileWithRiverAndTree:
                    AddItem(5, 5, MapItem.Item.Nest);
                    AddLine(0, 39, 39, 0, MapItem.Item.Water, 2);
                    AddPile(27, 24, 30, 27, MapItem.Item.Food1);
                    AddPile(25, 3, 27, 5, MapItem.Item.Food1);
                    AddPile(3, 25, 5, 27, MapItem.Item.Food1);
                    AddItem(15, 20, MapItem.Item.Tree);
                    AddItem(20, 25, MapItem.Item.Tree);
                    break;
				case CannedMaps.SixFood5WithWallAndRiver:
                    AddItem(5, 5, MapItem.Item.Nest);
                    AddLine(22, 16, 39, 16, MapItem.Item.Obstacle, 2);
                    AddLine(0, 37, 32, 5, MapItem.Item.Water, 2);
                    AddItem(34, 3, MapItem.Item.Food9);
                    AddItem(12, 12, MapItem.Item.Food9);
                    AddItem(35, 12, MapItem.Item.Food9);
                    AddItem(3, 20, MapItem.Item.Food9);
                    AddItem(35, 23, MapItem.Item.Food9);
                    AddItem(25, 31, MapItem.Item.Food9);
					break;
			}
		}

        /// <summary>
        /// Draw the pheromone strengths on the image object. Note, pheromones should be drawn on the map first.
        /// </summary>
        /// <param name="imageSize">The size of the image canvas to draw the pheromones on.</param>
        /// <returns>The image depicting pheromone strengths.</returns>
        private Image DrawPheromones(Size imageSize)
        {
            Image img = new Bitmap(imageSize.Width, imageSize.Height);
            Graphics graphics = Graphics.FromImage(img);
            Size itemSize = new Size((int)(imageSize.Width / mapSize.Width), (int)(imageSize.Height / mapSize.Height));
            int colorValue;
            Color color;

            graphics.FillRectangle(Brushes.White, 0, 0, imageSize.Width, imageSize.Height);

            for (int col = 0; col < mapSize.Width; col++)
            {
                for (int row = 0; row < mapSize.Height; row++)
                {
                    colorValue = (int)(256.0 * (double)pheromones[col, row] / 5000.0);
                    if (colorValue > 255)
                    {
                        colorValue = 255;
                    }
                    if ((colorValue <= 5) && (pheromones[col, row] > 0))
                    {
                        colorValue = 5;
                    }
                    colorValue = 255 - colorValue;
                    color = Color.FromArgb(colorValue, colorValue, colorValue);
                    using (Brush brush = new SolidBrush(color))
                    {
                        graphics.FillRectangle(brush, itemSize.Width * col, itemSize.Height * row, itemSize.Width, itemSize.Height);
                    }
                }
            }

            return img;
        }
        
        /// <summary>
        /// Draw the map contents on an image.
        /// </summary>
        /// <param name="imageSize">The size of the image canvas.</param>
        /// <returns>The image depicting pheromone strengths and map items.</returns>
		public Image DrawMap(Size imageSize)
		{
            Image img = DrawPheromones(imageSize);
            Graphics graphics = Graphics.FromImage(img);
            SizeF itemSizeFloat = new SizeF(((float)imageSize.Width / (float)mapSize.Width), ((float)imageSize.Height / (float)mapSize.Height));
            Size itemSize = new Size((int)itemSizeFloat.Width, (int)itemSizeFloat.Height);
			Image icon;
            Food food;

			for ( int col = 0; col < mapSize.Width; col++ )
			{
				for ( int row = 0; row < mapSize.Height; row++ )
				{
                    food = FoodAtLocation(new Point(col, row));
                    if (food != null)
                    {
                        if (food.ItemUnderFood != MapItem.Item.Nothing)
                        {
                            icon = icons.GetIcon(food.ItemUnderFood, itemSize);
                            graphics.DrawImage(icon, itemSizeFloat.Width * col, itemSizeFloat.Height * row);
                        }
                    }

					if ( mapItems[ col, row ] != MapItem.Item.Nothing )
					{
						icon = icons.GetIcon( mapItems[col, row], itemSize );
                        graphics.DrawImage(icon, itemSizeFloat.Width * col, itemSizeFloat.Height * row);
					}
				}
			}

			return img;
		}

        /// <summary>
        /// Draw a map with pheromones, map items, and ants.
        /// </summary>
        /// <param name="imageSize">The size of the image canvas.</param>
        /// <param name="ants">The ants on the map.</param>
        /// <returns>The map image containing pheromones, map items, and ants.</returns>
        public Image DrawMap(Size imageSize, Colony ants)
        {
            Image img = DrawMap(imageSize);
            Graphics graphics = Graphics.FromImage(img);
            Size itemSize = new Size( (int)( imageSize.Width / mapSize.Width ), (int) ( imageSize.Height / mapSize.Height ));
            Image icon;

            foreach (Ant ant in ants)
            {
                MapItem.Item antType = MapItem.Item.Ant;
                if (ant.FoodCarried != null)
                {
                    antType = MapItem.Item.AntWithFood;
                }
                if (ant.CarryingLeaf)
                {
                    antType = MapItem.Item.AntWithLeaf;
                }
                if (!ant.IsAlive)
                {
                    continue;
                }
                icon = icons.GetIcon(antType, itemSize);
                switch(ant.Direction)
                {
                    case Compass.CompassDirection.East: icon.RotateFlip(RotateFlipType.Rotate90FlipNone);
                        break;
                    case Compass.CompassDirection.South: icon.RotateFlip(RotateFlipType.Rotate180FlipNone);
                        break;
                    case Compass.CompassDirection.West: icon.RotateFlip(RotateFlipType.Rotate270FlipNone);
                        break;
                }
                graphics.DrawImage(icon, itemSize.Width * ant.Location.X, itemSize.Height * ant.Location.Y);
            }

            return img;
        }
	}
}
