//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: Tokens.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Reflection;

namespace AntGP
{
	/// <summary>
	/// This class handles the instruction set for the ant programs.
	/// </summary>
	public class Tokens
	{
        /// <summary>
        /// The list of all allowable instructions for the ant programs.
        /// </summary>
		public enum Instruction
		{
			/////////// LEAF STATEMENTS ////////////////

            /// <summary>Turn in a random direction and move one square forward.</summary>
			MoveRandom,
            /// <summary>Turn in a pseudo-random direction (greatest chance face forward, least chance full 180) and  move one square forward.</summary>
			MoveQuasiRandom,
            /// <summary>Move one square in the direction of the nest.</summary>
			MoveToNest,
            /// <summary>Move one square in the direction of the nest. If run into water, then move into the water and die and build an ant bridge.</summary>
			MoveToNestThroughWater,
            /// <summary>Turn left 90 degrees.</summary>
			TurnLeft,
            /// <summary>Turn right 90 degrees.</summary>
			TurnRight,
            /// <summary>Move forward one square.</summary>
			MoveForward,
            /// <summary>If there is food at the current location, pick it up.</summary>
			PickUpFood,
            /// <summary>If possible, drop food at the current location. Food cannot be dropped on other food or on a tree or a leaf.</summary>
			DropFood,
            /// <summary>Pick up a leaf, either from a tree or a leaf an ant left on the ground.</summary>
            PickUpLeaf,
            /// <summary>If carried, drop a leaf on the ground.</summary>
            DropLeaf,
            /// <summary>If there is water in the square in front of the ant, and if a leaf is carried, drop the leaf into the water.</summary>
            DropLeafInWater,
            /// <summary>Release pheromones at the current location.</summary>
			ReleasePheromone,
            /// <summary>If there is water in front of the ant, then move into the water and die and build an ant bridge.</summary>
			MoveIntoWater,
            /// <summary>Move forward until there are no pheromones around. If hit an obstacle, turn in a random direction and continue to move.</summary>
			EscapePheromone,
            /// <summary>Do nothing operation.</summary>
			NoOp,

            /////////// NON-LEAF STATEMENTS ////////////////

            /// <summary>Used to execute more than one statement in a if or else block.</summary>
			TwoStatements,
            /// <summary>Is there food in the square directly in front of the ant.</summary>
			IfFoodAhead,
            /// <summary>Is there food in the square to the left of the ant.</summary>
			IfFoodLeft,
            /// <summary>Is there food in the square to the right of the ant.</summary>
			IfFoodRight,
            /// <summary>Is there food in any of the 4 squares adjacent to the ant.</summary>
			IfFoodAdjacent,
            /// <summary>Is there food in the same square as the ant.</summary>
			IfFoodHere,
            /// <summary>Is there any pheromones in the same square as the ant.</summary>
            IfPheromoneHere,
            /// <summary>Is there a leaf in the same square as the ant.</summary>
            IfLeafHere,
            /// <summary>Is the ant carrying food.</summary>
			IfCarryingFood,
            /// <summary>Is the ant carrying a leaf.</summary>
            IfCarryingLeaf,
            /// <summary>Is the ant at the nest.</summary>
			IfAtNest,
            /// <summary>Is there any food in the 4 squares adjacent to the ant. Side effect - move to that square.</summary>
			IfMoveToAdjacentFood,
            /// <summary>Is there a tree or a leaf in the 4 squares adjacent to the ant. Side effect - move to that square.</summary>
            IfMoveToAdjacentLeaf,
            /// <summary>Are there pheromones in the 2 squares that are away from the nest. Side effect - move to the square with the strongest pheromone.</summary>
			IfMoveToPheromoneAwayFromNest,
            /// <summary>Is there a dead ant in the 4 squares adjacent to the ant. Side effect - move to that square.</summary>
			IfMoveToDeadAnt,
            /// <summary>Are there pheromones in the 4 squares adjacent to the ant. Side effect - move to the square with the strongest pheromone.</summary>
			IfMoveToStrongestPheromone,
            /// <summary>Is it possible for this ant to move toward the nest (false if food is too heavy, or water or obstacle blocks the path). Side effect - move toward the nest.</summary>
			IfMoveToNest,
            /// <summary>Is it possible for this ant to move toward the nest (false if food is too heavy or obstacle blocks the path). Side effect - move toward the nest and if there is water in the way move into the water and die and build an ant bridge.</summary>
			IfMoveToNestThroughWater,
            /// <summary>Is there water in the square in front of the ant. Squares with dead ants or leaves do not count.</summary>
			IfWaterAhead,
            /// <summary>Is there an obstacle in the square in front of the ant.</summary>
			IfObstacleAhead,
            /// <summary>Is there another ant at the same location as this ant.</summary>
			IfAnotherAntHere,
            /// <summary>Is this ant lifting food and there are not enough ants to help lift the food.</summary>
			IfCantLiftFood,
            /// <summary>Is this ant trying to lift food that is to heavy, and through a random number generator the ant has decided to give up.</summary>
			IfTiredOfWaiting
		}

        /// <summary>
        /// Is the given instruction a leaf node (no children).
        /// </summary>
        /// <param name="instruction">The instruction to examine.</param>
        /// <returns>Is this instruction a leaf node.</returns>
		public static bool IsLeafNode( Instruction instruction )
		{
			if ((instruction.ToString().IndexOf("If") == 0 ) || (instruction == Instruction.TwoStatements))
			{
				return false;
			}
			else
			{
				return true;
			}
		}

        /// <summary>
        /// Is this instruction an if statement.
        /// </summary>
        /// <param name="instruction">The instruction to examine - as a string.</param>
        /// <returns>Is this instruction an if statement.</returns>
        public static bool IsIfStatement(String instruction)
        {
            if (instruction.ToLower().IndexOf("if") == 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Is this instruction an if statement.
        /// </summary>
        /// <param name="instruction">The instruction to examine.</param>
        /// <returns>Is this instruction an if statement.</returns>
        public static bool IsIfStatement(Instruction instruction)
        {
            return IsIfStatement(instruction.ToString());
        }

        /// <summary>
        /// Is this instruction an else statement.
        /// Note: There is no real ELSE statement token in the binary tree.
        /// </summary>
        /// <param name="instruction">The instruction to examine - as a string.</param>
        /// <returns>Is this instruction an else statement.</returns>
        public static bool IsElseStatement(string instruction)
        {
            return instruction.ToLower().Equals("else");
        }

        /// <summary>
        /// Convert an instruction (the enum value as a string) to a displayable instruction string.
        /// Ex: IfFoodHere converts to if ( FoodHere )
        /// </summary>
        /// <param name="instruction">The instruction to convert.</param>
        /// <returns>The instruction in a nice display format.</returns>
        public static string DisplayableInstruction(string instruction)
        {
            if (instruction.IndexOf("If") == 0)
            {
                return ("if ( " + instruction.Substring(2) + " )");
            }
            else if (instruction.Equals("TwoStatements"))
            {
                return "Two Statements ()";
            }
            else
            {
                return instruction.ToString() + ";";
            }
        }

        /// <summary>
        /// Convert an instruction to a displayable instruction string.
        /// Ex: IfFoodHere converts to if ( FoodHere )
        /// </summary>
        /// <param name="instruction">The instruction to convert.</param>
        /// <returns>The instruction in a nice display format.</returns>
        public static string DisplayableInstruction(Instruction instruction)
        {
            return DisplayableInstruction(instruction.ToString());
        }
        
        /// <summary>
        /// Converts an instruction string to an instruction token.
        /// </summary>
        /// <param name="instructionName">The string name of the instruction.</param>
        /// <returns>The token for that instruction.</returns>
		public static Instruction StringToInstruction( string instructionName )
		{
			instructionName = instructionName.ToLower();

			foreach ( FieldInfo fi in typeof(Instruction).GetFields() )
			{
				if ( fi.Name.ToLower().Equals( instructionName ))
				{
					return (Instruction) fi.GetValue( null );
				}
			}

			return Instruction.NoOp;
		}

        /// <summary>
        /// Randomly selects a leaf instruction based on the list of legal leaf node instructions in the Genetic Program Options.
        /// </summary>
        /// <param name="options">The Genetic Program options containing the legal set of instructions to use and the random number generator.</param>
        /// <returns>The random leaf instruction.</returns>
        public static Instruction GetRandomLeafNode(GeneticProgramOptions options)
        {
            return options.LeafInstructionSet[options.InterpreterRandom(options.LeafInstructionSet.Count)];
        }

        /// <summary>
        /// Randomly selects a non-leaf instruction based on the list of legal leaf node instructions in the Genetic Program Options.
        /// </summary>
        /// <param name="options">The Genetic Program options containing the legal set of instructions to use and the random number generator.</param>
        /// <returns>The random non-leaf instruction.</returns>
        public static Instruction GetRandomNonLeafNode(GeneticProgramOptions options)
        {
            return options.NonLeafInstructionSet[options.InterpreterRandom(options.NonLeafInstructionSet.Count)];
        }
	}
}
